@@ -3,7 +3,7 @@
 """
 Job control on SGE clusters (possibly connecting to the front-end via SSH).
 """
-# Copyright (C) 2009-2013 GC3, University of Zurich. All rights reserved.
+# Copyright (C) 2009-2014 GC3, University of Zurich. All rights reserved.
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU Lesser General Public License as published by
@@ -20,9 +20,10 @@ Job control on SGE clusters (possibly connecting to the front-end via SSH).
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 #
 __docformat__ = 'reStructuredText'
-__version__ = '2.2.2 version (SVN $Revision$)'
+__version__ = '2.2.3 version (SVN $Revision$)'
 
 
+from collections import defaultdict
 import datetime
 import os
 import posixpath
@@ -367,6 +368,7 @@ class LsfLrms(batch.BatchSystem):
                  # these are inherited from `BatchSystem`
                  frontend, transport,
                  # these are specific to this backend
+                 lsf_continuation_line_prefix_length=None,
                  # (Note that optional arguments to the `BatchSystem` class, e.g.:
                  #     keyfile=None, accounting_delay=15,
                  # are collected into `extra_args` and should not be explicitly
@@ -389,6 +391,11 @@ class LsfLrms(batch.BatchSystem):
         self._bkill = self._get_command('bkill')
         self._lshosts = self._get_command('lshosts')
 
+        if lsf_continuation_line_prefix_length is not None:
+            self._CONTINUATION_LINE_START = ' ' * lsf_continuation_line_prefix_length
+        else:
+            self._CONTINUATION_LINE_START = None
+
 
     def _submit_command(self, app):
         # LSF's `bsub` allows one to submit scripts and binaries with
@@ -440,21 +447,24 @@ class LsfLrms(batch.BatchSystem):
     _unsuccessful_exit_re = re.compile(r'Exited with exit code (?P<exit_status>[0-9]+).', re.M)
     _cpu_time_re = re.compile(r'The CPU time used is (?P<cputime>[0-9]+(\.[0-9]+)?) seconds', re.M)
 
-    _CONTINUATION_LINE_START = '                     '
-
-    @staticmethod
-    def _parse_stat_output(stdout):
+    def _parse_stat_output(self, stdout):
         # LSF `bjobs -l` uses a LDIF-style continuation lines, wherein
         # a line is truncated at 79 characters and continues upon the
         # next one; continuation lines start with a fixed amount of
-        # whitespace.  Join continuation lines, so that we can work on
-        # a single block of text.
+        # whitespace.  However, the amount of whitespace varies with
+        # LSF release and possibly other factors, so we need to guess
+        # or have users configure it...
+        if self._CONTINUATION_LINE_START is None:
+            self._CONTINUATION_LINE_START = ' ' * self._guess_continuation_line_prefix_len(stdout)
+
+        # Join continuation lines, so that we can work on a single
+        # block of text.
         lines = [ ]
         for line in stdout.split('\n'):
             if len(line) == 0:
                 continue
-            if line.startswith(LsfLrms._CONTINUATION_LINE_START):
-                lines[-1] += line[len(LsfLrms._CONTINUATION_LINE_START):]
+            if line.startswith(self._CONTINUATION_LINE_START):
+                lines[-1] += line[len(self._CONTINUATION_LINE_START):]
             else:
                 lines.append(line)
 
@@ -480,27 +490,65 @@ class LsfLrms(batch.BatchSystem):
         assert 'state' in jobstatus
         return jobstatus
 
-    _TIMESTAMP_FMT = '%a %b %d %H:%M:%S' # e.g., 'Mon Oct  8 12:04:56 2012'
+    @staticmethod
+    def _guess_continuation_line_prefix_len(stdout):
+        """
+        Guess the most likely length of the initial run of spaces in
+        continuation lines in `bjobs` output.
+
+        The euristics is rather crude: we count how many spaces are at
+        the beginning of each line, and take the value with most
+        occurrences.
+
+        Since, in addition, LSF's `bjobs` output contains also
+        scheduling parameters and other differently-formatted reports,
+        we only consider lines that contain a ``<`` or a ``>``
+        character to be valid continuation lines -- any other line is
+        just discarded.
+
+        This is necessary as the amount of whitespace at the beginning
+        of lines seems to vary with LSF version and/or some other
+        parameter.
+        """
+        # count occurrences of each prefix length
+        occurrences = defaultdict(int)
+        for line in stdout.split('\n'):
+            if '<' not in line and '>' not in line:
+                continue
+            # FIXME: incorrect result if LSF mixes TABs and spaces
+            ws_length = len(line) - len(line.lstrip())
+            occurrences[ws_length] += 1
+        # now find the length that has the max occurrences
+        max_occurrences = max(occurrences.values())
+        for length, count in occurrences.items():
+            if count == max_occurrences:
+                return length
+
+    _TIMESTAMP_FMT_WITH_YEAR = '%a %b %d %H:%M:%S %Y' # e.g., 'Mon Oct  8 12:04:56 2012'
+    _TIMESTAMP_FMT_NO_YEAR   = '%a %b %d %H:%M:%S'    # e.g., 'Mon Oct  8 12:04:56'
 
     @staticmethod
     def _parse_timespec(ts):
         """Parse a timestamp as it appears in LSF bjobs/bacct logs."""
+        # try "with year" format first, as it has all the info we need
         try:
-            # XXX: I couldn't find examples of LSF output that
-            # contain a year spec, so we do the following: if the
-            # month in the timespec is less than or equal to the
-            # current month, the timestamp is for an event occurred
-            # during this year; if it's in a month later than the
-            # current one, the timestamp refers to an event occurred
-            # in the *past* year.
+            return datetime.datetime.strptime(ts, LsfLrms._TIMESTAMP_FMT_WITH_YEAR)
+        except ValueError:
+            pass # ignore and try again without year
+        try:
+            # XXX: since we do not have a year, we resort to the
+            # following heuristics: if the month in the timespec is
+            # less than or equal to the current month, the timestamp
+            # is for an event occurred during this year; if it's in a
+            # month later than the current one, the timestamp refers
+            # to an event occurred in the *past* year.
             today = datetime.date.today()
             # XXX: datetime.strptime() only available starting Py 2.5
-            tm = time.strptime(ts, LsfLrms._TIMESTAMP_FMT)
+            tm = time.strptime(ts, LsfLrms._TIMESTAMP_FMT_NO_YEAR)
             if tm[1] <= today.month:
-                return datetime.datetime(today.year, *(time.strptime(ts, LsfLrms._TIMESTAMP_FMT)[1:6]))
+                return datetime.datetime(today.year, *(tm[1:6]))
             else:
-                return datetime.datetime(today.year-1,
-                                         *(time.strptime(ts, LsfLrms._TIMESTAMP_FMT)[1:6]))
+                return datetime.datetime(today.year-1, *(tm[1:6]))
         except ValueError, err:
             gc3libs.log.error(
                 "Cannot parse '%s' as an LSF timestamp: %s: %s",
