@@ -20,7 +20,7 @@ Implementation of the `core` command-line front-ends.
 # Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 #
 __docformat__ = 'reStructuredText'
-__version__ = '2.2.2 version (SVN $Revision: 3979 $)'
+__version__ = '2.2.3 version (SVN $Revision: 4001 $)'
 __author__ = "Sergio Maffioletti <sergio.maffioletti@gc3.uzh.ch>, Riccardo Murri <riccardo.murri@uzh.ch>, Antonio Messina <arcimboldo@gmail.com>"
 __date__ = '$Date$'
 __copyright__ = "Copyright (c) 2009-2012 Grid Computing Competence Center, University of Zurich"
@@ -652,11 +652,19 @@ released once the output files have been fetched.
                 # wrap an application (e.g., `RetryableTask`)
                 if app_download_dir not in download_dirs:
                     self._core.fetch_output(app,
-                                            output_dir=self.params.download_dir,
+                                            download_dir=app_download_dir,
                                             overwrite=self.params.overwrite,
                                             changed_only=self.params.changed_only)
                     if app.changed:
                         self.session.store.replace(app.persistent_id, app)
+                    # `fetch_output` is by default a no-op on tasks:
+                    # try to detect this and skip all the messaging
+                    # below (and also retry the directory, in case we
+                    # have an Application with the same download
+                    # destination)
+                    if not os.path.exists(app_download_dir):
+                        continue
+                    # avoid downloading files twice -- see above
                     download_dirs.add(app_download_dir)
                 else:
                     self.log.debug("Output directory '%s' already visited,"
